@@ -1,41 +1,25 @@
 # Before `make install' is performed this script should be runnable with
 # `make test'. After `make install' it should work as `perl test.pl'
 
-######################### We start with some black magic to print on failure.
+use strict;
+use Test;
+BEGIN { plan tests => 43 };
 
-# Change 1..1 below to 1..last_test_to_print .
-# (It may become useful if the test is moved to ./t subdirectory.)
-
-BEGIN { $| = 1; print "1..36\n"; }
-END {print "not ok 1\n" unless $loaded;}
 use HTML::Template;
-$loaded = 1;
-print "ok 1\n";
+ok(1);
 
-######################### End of black magic.
-
-# Insert your test code below (better if it prints "ok 13"
-# (correspondingly "not ok 13") depending on the success of chunk 13
-# of the test code):
+my ($output, $template, $result);
 
 # test a simple template
-my $template = HTML::Template->new(
+$template = HTML::Template->new(
                                    path => 'templates',
                                    filename => 'simple.tmpl',
                                    debug => 0
                                   );
 
 $template->param('ADJECTIVE', 'very');
-my $output =  $template->output;
-if ($output =~ /ADJECTIVE/) {
-  die "not ok 2\n";
-} elsif ($template->param('ADJECTIVE') ne 'very') {
-  die "not ok 2\n";
-} elsif ($output =~ /very/) {
-  print "ok 2\n";
-} else {
-  die "not ok 2 : $output\n";
-}
+$output =  $template->output;
+ok($output !~ /ADJECTIVE/ and $template->param('ADJECTIVE') eq 'very');
 
 # try something a bit larger
 $template = HTML::Template->new(
@@ -67,11 +51,7 @@ $template->param('LOGO_LINK', "dsfpkjdsfkgljdsfkglj");
 $template->param('PHOTO_LINK', "lsadfjlkfjdsgkljhfgklhasgh");
 
 $output = $template->output;
-if ($output =~ /<TMPL_VAR/) {
-  die "not ok 3\n";
-} else {
-  print "ok 3\n";
-}
+ok($output !~ /<TMPL_VAR/);
 
 # test a simple loop template
 $template = HTML::Template->new(
@@ -82,13 +62,7 @@ $template = HTML::Template->new(
 $template->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
 
 $output =  $template->output;
-if ($output =~ /ADJECTIVE_LOOP/) {
-  die "not ok 4\n$output";
-} elsif ($output =~ /really.*very/s) {
-  print "ok 4\n";
-} else {
-  die "not ok 4\n$output";
-}
+ok($output !~ /ADJECTIVE_LOOP/ and $output =~ /really.*very/s);
 
 # test a simple loop template
 $template = HTML::Template->new(
@@ -99,13 +73,7 @@ $template = HTML::Template->new(
 $template->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
 
 $output =  $template->output;
-if ($output =~ /ADJECTIVE_LOOP/) {
-  die "not ok 5\n\n";
-} elsif ($output =~ /really.*very/s) {
-  print "ok 5\n";
-} else {
-  die "not ok 5\n\n";
-}
+ok($output !~ /ADJECTIVE_LOOP/ and  $output =~ /really.*very/s);
 
 # test a long loop template - mostly here to use timing on.
 $template = HTML::Template->new(
@@ -114,7 +82,7 @@ $template = HTML::Template->new(
                                 # debug => 1,
                                );
 $output =  $template->output;
-print "ok 6\n";
+ok(1);
 
 # test a template with TMPL_INCLUDE
 $template = HTML::Template->new(
@@ -123,11 +91,7 @@ $template = HTML::Template->new(
                                 # debug => 1,
                                );
 $output =  $template->output;
-if (!($output =~ /5/) || !($output =~ /6/)) {
-  die "not ok 7\n";
-} else {
-  print "ok 7\n";
-}
+ok(not (!($output =~ /5/) || !($output =~ /6/)));
 
 # test a template with TMPL_INCLUDE and cacheing.
 $template = HTML::Template->new(
@@ -138,9 +102,7 @@ $template = HTML::Template->new(
                                 # debug => 1,
                                );
 $output =  $template->output;
-if (!($output =~ /5/) || !($output =~ /6/)) {
-  die "not ok 8\n";
-} 
+ok(not (!($output =~ /5/) || !($output =~ /6/)));
 
 # stimulates a cache miss
 # system('touch templates/included2.tmpl');
@@ -153,11 +115,7 @@ my $template2 = HTML::Template->new(
                                     # debug => 1,
                                    );
 $output =  $template->output;
-if (!($output =~ /5/) || !($output =~ /6/)) {
-  die "not ok 8\n";
-} else {
-  print "ok 8\n";
-}
+ok(not (!($output =~ /5/) || !($output =~ /6/)));
 
 # test associate
 my $template_one = HTML::Template->new(
@@ -175,13 +133,7 @@ my $template_two = HTML::Template->new (
                                        );
 
 $output =  $template_two->output;
-if ($output =~ /ADJECTIVE/) {
-  die "not ok 9\n";
-} elsif ($output =~ /very/) {
-  print "ok 9\n";
-} else {
-  die "not ok 9\n";
-}
+ok($output !~ /ADJECTIVE/ and $output =~ /very/);
 
 # test a simple loop template
 my $template_l = HTML::Template->new(
@@ -192,12 +144,7 @@ my $template_l = HTML::Template->new(
 # $template_l->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
 
 $output =  $template_l->output;
-if ($output =~ /INSIDE/) {
-  die "not ok 10\n";
-} else {
-  print "ok 10\n";
-}
-
+ok($output !~ /INSIDE/);
 
 # test a simple if template
 my $template_i = HTML::Template->new(
@@ -208,11 +155,7 @@ my $template_i = HTML::Template->new(
 # $template_l->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
 
 $output =  $template_i->output;
-if ($output =~ /INSIDE/) {
-  die "not ok 11\n";
-} else {
-  print "ok 11\n";
-}
+ok($output !~ /INSIDE/);
 
 # test a simple if template
 my $template_i2 = HTML::Template->new(
@@ -223,12 +166,7 @@ my $template_i2 = HTML::Template->new(
 $template_i2->param(BOOL => 1);
 
 $output =  $template_i2->output;
-if ($output !~ /INSIDE/) {
-  die "not ok 12\n";
-} else {
-  print "ok 12\n";
-}
-
+ok($output =~ /INSIDE/);
 
 # test a simple if/else template
 my $template_ie = HTML::Template->new(
@@ -238,13 +176,7 @@ my $template_ie = HTML::Template->new(
                                      );
 
 $output =  $template_ie->output;
-if ($output !~ /INSIDE ELSE/) {
-  die "not ok 13\n";
-} elsif ($output =~ /INSIDE IF/) {
-  die "not ok 13\n";
-} else {
-  print "ok 13\n";
-}
+ok ($output =~ /INSIDE ELSE/);
 
 # test a simple if/else template
 my $template_ie2 = HTML::Template->new(
@@ -255,13 +187,7 @@ my $template_ie2 = HTML::Template->new(
 $template_ie2->param(BOOL => 1);
 
 $output =  $template_ie2->output;
-if ($output !~ /INSIDE IF/) {
-  die "not ok 14\n";
-} elsif ($output =~ /INSIDE ELSE/) {
-  die "not ok 14\n";
-} else {
-  print "ok 14\n";
-}
+ok($output =~ /INSIDE IF/ and $output !~ /INSIDE ELSE/);
 
 # test a bug involving two loops with the same name
 $template = HTML::Template->new(
@@ -276,11 +202,7 @@ $template->param('myloop', [
                            ]
                 );
 $output = $template->output;
-if ($output !~ /David/) {
-  die "not ok 15\n";
-} else {
-  print "ok 15\n";
-}
+ok($output =~ /David/);
 
 # test escapeing
 $template = HTML::Template->new(
@@ -290,12 +212,7 @@ $template = HTML::Template->new(
                                );
 $template->param(STUFF => '<>"\''); #"
 $output = $template->output;
-if ($output =~ /[<>"']/) { #"
-  die "not ok 16\n";
-} else {
-  print "ok 16\n";
-}
-
+ok($output !~ /[<>"']/); #"
 
 # test a simple template, using new param() call format
 $template = HTML::Template->new(
@@ -310,13 +227,7 @@ $template->param(
                  }
                 );
 $output =  $template->output;
-if ($output =~ /ADJECTIVE/) {
-  die "not ok 2\n";
-} elsif ($output =~ /very/) {
-  print "ok 17\n";
-} else {
-  die "not ok 17\n";
-}
+ok($output !~ /ADJECTIVE/ and $output =~ /very/);
 
 # test a recursively including template
 eval {
@@ -328,11 +239,7 @@ eval {
   $output =  $template->output;
 };
 
-if (defined($@) and ($@ =~ /recursive/)) {
-  print "ok 18\n";
-} else {
-  print "not ok 18\n";
-}
+ok(defined($@) and ($@ =~ /recursive/));
 
 # test a template using unless
 $template = HTML::Template->new(
@@ -343,13 +250,7 @@ $template = HTML::Template->new(
 $template->param(BOOL => 1);
 
 $output =  $template->output;
-if ($output =~ /INSIDE UNLESS/) {
-  die "not ok 19\n";
-} elsif ($output !~ /INSIDE ELSE/) {
-  die "not ok 19\n";
-} else {
-  print "ok 19\n";
-}
+ok($output !~ /INSIDE UNLESS/ and $output =~ /INSIDE ELSE/);
 
 # test a template using unless
 $template = HTML::Template->new(
@@ -361,14 +262,7 @@ $template = HTML::Template->new(
 $template->param(BOOL => 0);
 
 $output =  $template->output;
-if ($output !~ /INSIDE UNLESS/) {
-  die "not ok 20\n";
-} elsif ($output =~ /INSIDE ELSE/) {
-  die "not ok 20\n";
-} else {
-  print "ok 20\n";
-}
-
+ok($output =~ /INSIDE UNLESS/ and $output !~ /INSIDE ELSE/);
 
 # test a template using loop_context_vars
 $template = HTML::Template->new(
@@ -388,14 +282,7 @@ $template->param(FRUIT => [
 $template->param(PINGPONG => [ {}, {}, {}, {}, {}, {} ]);
 
 $output =  $template->output;
-if ($output !~ /Apples, Oranges, Brains, Toes, and Kiwi./) {
-  die "not ok 21\n";
-} elsif ($output !~ /pingpongpingpongpingpong/) {
-  die "not ok 21:\n$output\n";
-} else {
-  print "ok 21\n";
-}
-
+ok($output =~ /Apples, Oranges, Brains, Toes, and Kiwi./ and $output =~ /pingpongpingpongpingpong/);
 
 $template = HTML::Template->new(
                                 path => 'templates',
@@ -404,12 +291,7 @@ $template = HTML::Template->new(
                                 #debug_stack => 1
                                );
 $output =  $template->output;
-if ($output !~ /Loop not filled in/) {
-  die "not ok 22\n";
-} else {
-  print "ok 22\n";
-}
-
+ok($output =~ /Loop not filled in/);
 
 $template = HTML::Template->new(
                                 path => 'templates',
@@ -419,18 +301,14 @@ $template = HTML::Template->new(
                                );
 $template->param(LOOP_ONE => [{VAR => "foo"}]);
 $output =  $template->output;
-if ($output =~ /Loop not filled in/) {
-  die "not ok 23\n";
-} elsif ($output !~ /foo/) {
-  die "not ok 23\n";
-} else {
-  print "ok 23\n";
-}
+ok($output !~ /Loop not filled in/);
 
 # test shared memory - enable by setting the environment variable
 # TEST_SHARED_MEMORY to 1.
 if (!exists($ENV{TEST_SHARED_MEMORY}) or !$ENV{TEST_SHARED_MEMORY}) {
-  print "skipped 24 - shared memory cache test.  See README to enable.\n";
+  print "Skipping shared memory cache test.  See README to enable\n";
+  skip(1, 0); 
+  skip(1, 0);
 } else {
   require 'IPC/SharedCache.pm';
   my $template_prime = HTML::Template->new(
@@ -450,13 +328,7 @@ if (!exists($ENV{TEST_SHARED_MEMORY}) or !$ENV{TEST_SHARED_MEMORY}) {
                                     );
   $template->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
   $output =  $template->output;
-  if ($output =~ /ADJECTIVE_LOOP/) {
-    die "not ok 24";
-  } elsif ($output !~ /really.*very/s) {
-    die "not ok 24";
-  } else {
-    print "ok 24.1\n";
-  }
+  ok($output !~ /ADJECTIVE_LOOP/ and $output =~ /really.*very/s);
 
    my $template_prime2 = HTML::Template->new(
                                             filename => 'simple-loop.tmpl',
@@ -475,13 +347,7 @@ if (!exists($ENV{TEST_SHARED_MEMORY}) or !$ENV{TEST_SHARED_MEMORY}) {
                                      );
    $template->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
    $output =  $template->output;
-   if ($output =~ /ADJECTIVE_LOOP/) {
-     die "not ok 24";
-   } elsif ($output =~ /really.*very/s) {
-     print "ok 24.2\n";
-   } else {
-     die "not ok 24";
-   }
+   ok($output !~ /ADJECTIVE_LOOP/ and $output =~ /really.*very/s);
 
   IPC::SharedCache::remove('TEST');
 }
@@ -489,24 +355,19 @@ if (!exists($ENV{TEST_SHARED_MEMORY}) or !$ENV{TEST_SHARED_MEMORY}) {
 # test CGI associate bug    
 eval { require 'CGI.pm'; };
 if ($@) {
-  print "skipped 25 - need CGI.pm to test associate\n";
+  print "Skipping associate tests, need CGI.pm to test associate\n";
+  skip(1, 0); 
 } else {
   my $query = CGI->new('');
   $query->param('AdJecTivE' => 'very');
-  my $template = HTML::Template->new(
+  $template = HTML::Template->new(
                                      path => 'templates',
                                      filename => 'simple.tmpl',
                                      debug => 0,
                                      associate => $query,
                                     );
-  my $output =  $template->output;
-  if ($output =~ /ADJECTIVE/) {
-    die "not ok 25\n";
-  } elsif ($output =~ /very/) {
-    print "ok 25\n";
-  } else {
-    die "not ok 25\n";
-  }
+  $output =  $template->output;
+  ok($output =~ /very/);
 }
 
 # test subroutine as VAR
@@ -517,13 +378,7 @@ $template = HTML::Template->new(
                                );
 $template->param(ADJECTIVE => sub { return 'v' . '1e' . '2r' . '3y'; });
 $output =  $template->output;
-if ($output =~ /ADJECTIVE/) {
-  die "not ok 26\n";
-} elsif ($output =~ /v1e2r3y/) {
-  print "ok 26\n";
-} else {
-  die "not ok 26\n";
-}
+ok($output =~ /v1e2r3y/);
 
 # test cache - non automated, requires turning on debug watching STDERR!
 $template = HTML::Template->new(
@@ -542,13 +397,7 @@ $template = HTML::Template->new(
                                 # cache_debug => 1,
                                 debug => 0,
                                );
-if ($output =~ /ADJECTIVE/) {
-  die "not ok 27\n";
-} elsif ($output =~ /v1e2r3y/) {
-  print "ok 27\n";
-} else {
-  die "not ok 27\n";
-}
+ok($output =~ /v1e2r3y/);
 
 # test URL escapeing
 $template = HTML::Template->new(
@@ -559,54 +408,28 @@ $template = HTML::Template->new(
                                );
 $template->param(STUFF => '<>"; %FA'); #"
 $output = $template->output;
-if ($output =~ /[<>"]/) { #"
-  print $output;
-  die "not ok 28\n";
-} else {
-  print "ok 28\n";
-}
+ok($output !~ /[<>"]/); #"
 
 # test query()
 $template = HTML::Template->new(
                                 path => 'templates',
                                 filename => 'query-test.tmpl',
                                );
-if ($template->query(name => 'var') ne 'VAR') {
-  print STDERR "\$template->query(name => 'var') returned ", $template->query(name => 'var'), "\n";
-  die "not ok 29\n";
-}
-if ($template->query(name => 'EXAMPLE_LOOP') ne 'LOOP') {
-  print STDERR "\$template->query(name => 'EXAMPLE_LOOP') returned ", $template->query(name => 'EXAMPLE_LOOP'), "\n";
-  die "not ok 29\n";
-}
-
 my %params = map {$_ => 1} $template->query(loop => 'EXAMPLE_LOOP');
-unless (exists $params{bee}) {
-  die "not ok 29\n";
-}
-unless (exists $params{bop}) {
-  die "not ok 29\n";
-}
-unless (exists $params{example_inner_loop}) {
-  die "not ok 29\n";
-}
-if ($template->query(name => ['EXAMPLE_LOOP', 'EXAMPLE_INNER_LOOP']) ne 'LOOP'){
-  use Data::Dumper;
-  print STDERR Data::Dumper::Dumper(\($template->query(name => ['EXAMPLE_LOOP', 'EXAMPLE_INNER_LOOP']))), "\n";
-
-  die "not ok 29\n";
-}
 
 my @result;
 eval {
   @result = $template->query(loop => ['EXAMPLE_LOOP', 'BEE']);
 };
-if ($@ !~ /error/) {
-  die "not ok 29!", join(', ', $result[0]), ".\n";
-}
-
-print "ok 29\n";
 
+ok($@ =~ /error/ and
+   $template->query(name => 'var') eq 'VAR' and
+   $template->query(name => 'EXAMPLE_LOOP') eq 'LOOP' and
+   exists $params{bee} and
+   exists $params{bop} and
+   exists $params{example_inner_loop} and
+   $template->query(name => ['EXAMPLE_LOOP', 'EXAMPLE_INNER_LOOP']) eq 'LOOP'
+  );   
 
 # test query()
 $template = HTML::Template->new(                                
@@ -614,13 +437,9 @@ $template = HTML::Template->new(
                                 filename => 'query-test2.tmpl',
                                );
 my %p = map {$_ => 1} $template->query(loop => ['LOOP_FOO', 'LOOP_BAR']);
-unless (exists $p{foo} and exists $p{bar} and exists $p{bash}) {
-  die "not ok 30\n";
-}
-print "ok 30\n";
+ok(exists $p{foo} and exists $p{bar} and exists $p{bash});
 
 # test global_vars
-
 $template = HTML::Template->new(                                
                                 path => 'templates',
                                 filename => 'globals.tmpl',
@@ -629,12 +448,8 @@ $template = HTML::Template->new(
 $template->param(outer_loop => [{loop => [{'LOCAL' => 'foo'}]}]);
 $template->param(global => 'bar');
 $template->param(hidden_global => 'foo');
-
 $result = $template->output();
-unless ($result =~ /foobar/) {
-  die "not ok 31\n";
-}
-print "ok 31\n";
+ok($result =~ /foobar/);
 
 $template = HTML::Template->new(                                
                                 path => 'templates',
@@ -644,13 +459,7 @@ $template = HTML::Template->new(
 $template->param(FOO => 'bar');
 $template->param(BAZ => 'bop');
 $result = $template->output();
-unless ($result =~ /bar/) {
-  die "not ok 32\n";
-} 
-unless ($result =~ /bop/) {
-  die "not ok 32 :\n $result\n";
-} 
-print "ok 32\n";
+ok($result =~ /bar/ and $result =~ /bop/);
 
 $template = HTML::Template->new(                           
                                 path => 'templates',
@@ -659,11 +468,7 @@ $template = HTML::Template->new(
                                );
 $template->param(TEST_LOOP => [ { NUM => 1 } ]);
 $result = $template->output();
-if ($result !~ /1:FIRST::LAST:ODD/) {
-  die "not ok 33 :\n$result\n";
-}
-print "ok 33\n";
-
+ok($result =~ /1:FIRST::LAST:ODD/);
 
 # test a TMPL_INCLUDE from a later path directory back up to an earlier one
 # when using the search_path_on_include option
@@ -673,11 +478,7 @@ $template = HTML::Template->new(
                                 filename => 'include.tmpl',
                                );
 $output =  $template->output;
-if (!($output =~ /9/) || !($output =~ /6/)) {
-  die "not ok 34\n";
-} else {
-  print "ok 34\n";
-}
+ok($output =~ /9/ and $output =~ /6/);
 
 # test no_includes
 eval {
@@ -687,15 +488,13 @@ eval {
                                   no_includes => 1,
                                  );
 };
-if (not defined $@ or $@ !~ /no_includes/) {
-  die "not ok 35\n";
-} else {
-  print "ok 35\n";
-}
+ok(defined $@ and $@ =~ /no_includes/);
 
 # test file cache - non automated, requires turning on debug watching STDERR!
 if (!exists($ENV{TEST_FILE_CACHE}) or !$ENV{TEST_FILE_CACHE}) {
-  print "skipped 36 - file cache test.  See README to enable.\n";
+  print "Skipping file cache test.  See README to enable.\n";
+  skip(1,0);
+  skip(1,0);
 } else {
   $template = HTML::Template->new(
                                   path => ['templates/'],
@@ -715,14 +514,9 @@ if (!exists($ENV{TEST_FILE_CACHE}) or !$ENV{TEST_FILE_CACHE}) {
                                   #cache_debug => 1,
                                   #debug => 0,
                                  );
-  if ($output =~ /ADJECTIVE/) {
-    die "not ok 36\n";
-  } elsif ($output =~ /3y/) {
-    # print "ok 36\n";
-  } else {
-    die "not ok 36\n";
-  }
+  ok($output =~ /3y/);
 
+  my $x;
   $template = HTML::Template->new(filename => 'templates/simple-loop.tmpl',
                                   filter => {
                                              sub => sub {
@@ -740,19 +534,12 @@ if (!exists($ENV{TEST_FILE_CACHE}) or !$ENV{TEST_FILE_CACHE}) {
                                  );
 $template->param('ADJECTIVE_LOOP', [ { ADJECTIVE => 'really' }, { ADJECTIVE => 'very' } ] );
   $output =  $template->output;
-  if ($output !~ /very/) {
-    die "not ok 36\n";
-  }
-  print "ok 36\n";
-
+  ok($output =~ /very/);
 }
 
 $template = HTML::Template->new(filename => './templates/include_path/a.tmpl');
 $output =  $template->output;
-if ($output !~ /Bar/) {
-  die "not ok 37\n";
-}
-print "ok 37\n";
+ok($output =~ /Bar/);
 
 open(OUT, ">blib/test.out") or die $!;
 $template = HTML::Template->new(filename => './templates/include_path/a.tmpl');
@@ -761,11 +548,7 @@ close(OUT);
 open(OUT, "blib/test.out") or die $!;
 my $output = join('',<OUT>);
 close(OUT);
-if ($output !~ /Bar/) {
-  die "not ok 38\n";
-}
-print "ok 38\n";
-
+ok($output =~ /Bar/);
 
 my $test = 39; # test with case sensitive params on
 my $template_source = <<END_OF_TMPL;
@@ -780,19 +563,12 @@ $template = HTML::Template->new(
 $template->param('adverb', 'very');
 $template->param('ADVERB', 'painfully');
 $output =  $template->output;
-if ($output =~ /ADVERB/i) {
-  die "not ok $test\n";
-} elsif ($template->param('ADVERB') ne 'painfully') {
-  die "not ok $test\n";
-} elsif ($template->param('adverb') ne 'very') {
-  die "not ok $test\n";
-} elsif ($output =~ /very painfully/) {
-  print "ok $test\n";
-} else {
-  die "not ok $test : $output\n";
-}
+ok($output !~ /ADVERB/i and 
+   $template->param('ADVERB') eq 'painfully' and
+   $template->param('adverb') eq 'very' and
+   $output =~ /very painfully/);
 
-$test = 40; # test with case sensitive params off
+# test with case sensitive params off
 $template_source = <<END_OF_TMPL;
   I am a <TMPL_VAR NAME="adverb"> <TMPL_VAR NAME="ADVERB"> simple template.
 END_OF_TMPL
@@ -805,18 +581,10 @@ $template = HTML::Template->new(
 $template->param('adverb', 'very');
 $template->param('ADVERB', 'painfully');
 $output =  $template->output;
-if ($output =~ /ADVERB/i) {
-  die "not ok $test\n";
-} elsif ($template->param('ADVERB') ne 'painfully') {
-  die "not ok $test\n";
-} elsif ($template->param('adverb') ne 'painfully') {
-  die "not ok $test\n";
-} elsif ($output =~ /painfully painfully/) {
-  print "ok $test\n";
-} else {
-  die "not ok $test : $output\n";
-}
-
+ok($output !~ /ADVERB/i and
+   $template->param('ADVERB') eq 'painfully' and
+   $template->param('adverb') eq 'painfully' and
+   $output =~ /painfully painfully/);
 
 $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                 filter => sub {
@@ -824,10 +592,7 @@ $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                 }
                                );
 $output =  $template->output;
-if ($output !~ /Zanzabar/) {
-  die "not ok 41\n";
-}
-print "ok 41\n";
+ok($output =~ /Zanzabar/);
 
 $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                 filter => [
@@ -846,11 +611,9 @@ $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                           ]
                                );
 $output =  $template->output;
-if ($output !~ /Zanzabar!!!/) {
-  die "not ok 42\n";
-}
-print "ok 42\n";
+ok($output =~ /Zanzabar!!!/);
 
+my $x;
 $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                 filter => {
                                            sub => sub {
@@ -864,8 +627,45 @@ $template = HTML::Template->new(filename => './templates/include_path/a.tmpl',
                                           }
                                );
 $output =  $template->output;
-if ($output !~ /1 : Foo/) {
-  die "not ok 43\n";
-}
-print "ok 43\n";
+ok($output =~ /1 : Foo/);
 
+$template = HTML::Template->new(
+                                scalarref => \ "\n<TMPL_INCLUDE templates/simple.tmpl>",
+                               );
+$template->param(ADJECTIVE => sub { "3y"; });
+$output = $template->output();
+ok($output =~ /3y/);
+
+$template = HTML::Template->new(path => ['templates'],
+                                filename => 'newline_test1.tmpl',
+                                filter => sub {},
+                               );
+$output = $template->output();
+ok($output =~ /STARTincludeEND/);
+
+# test multiline tags
+$template = HTML::Template->new(path => ['templates'],
+                                filename => 'multiline_tags.tmpl',
+                                global_vars => 1,
+                               );
+$template->param(FOO => 'foo!', bar => [{}, {}]);
+$output = $template->output();
+ok($output =~ /foo!\n/ && $output =~ /foo!foo!\nfoo!foo!/);
+
+# test new() from filehandle
+open(TEMPLATE, "templates/simple.tmpl");
+$template = HTML::Template->new(filehandle => *TEMPLATE);
+
+$template->param('ADJECTIVE', 'very');
+$output =  $template->output;
+ok($output !~ /ADJECTIVE/ and $template->param('ADJECTIVE') eq 'very');
+close(TEMPLATE);
+
+# test new_() from filehandle
+open(TEMPLATE, "templates/simple.tmpl");
+$template = HTML::Template->new_filehandle(*TEMPLATE);
+
+$template->param('ADJECTIVE', 'very');
+$output =  $template->output;
+ok($output !~ /ADJECTIVE/ and $template->param('ADJECTIVE') eq 'very');
+close(TEMPLATE);
